package com.cmb.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.chinapay.comm.Constants;
import com.chinapay.dto.HlsEbankCcbTransaction;
import com.chinapay.mapper.HlsEbankCcbTransactionMapper;
import com.chinapay.secss.LogUtil;
import com.cmb.dto.Ld037;
import com.cmb.mapper.Ld037Mapper;
import com.cmb.service.HclcCmbPolyService;
import com.cmb.util.MD5Utils;
import com.cmb.util.SM2Util;
import com.cmb.util.SignatureUtil;
import com.cmb.util.Utils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hand.hap.core.IRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
public class HclcCmbPolyServiceImpl implements HclcCmbPolyService {
    private static final String url = "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/qrcodeapply";  //UAT
    //uat环境商户国密私钥
    public static final String privateKey = "D5F2AFA24E6BA9071B54A8C9AD735F9A1DE9C4657FA386C09B592694BC118B38";
    //uat环境招行国密公钥
    public static final String publicKey = "MFkwEwYHKoZIzj0CAQYIKoEcz1UBgi0DQgAE6Q+fktsnY9OFP+LpSR5Udbxf5zHCFO0PmOKlFNTxDIGl8jsPbbB/9ET23NV+acSz4FEkzD74sW2iiNVHRLiKHg==";

    public static final String appId = "8ab74856-8772-45c9-96db-54cb30ab9f74";
    public static final String appSecret = "5b96f20a-011f-4254-8be8-9a5ceb2f317f";

    @Value("${cmb.version}")
    private String version;
    @Value("${cmb.encoding}")
    private String encoding;
    @Value("${cmb.signMethod}")
    private String signMethod;

    /*币种，非必传，默认156人民币*/
    @Value("${cmb.currencyCode}")
    private String currencyCode;

    @Autowired
    private HlsEbankCcbTransactionMapper hlsEbankCcbTransactionMapper;
    @Autowired
    private Ld037Mapper ld037Mapper;
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Override
    public JSONObject getQrcode(IRequest iRequest, JSONObject params) {
        JSONObject result = new JSONObject();

        if (Objects.isNull(params.get("recamt_id"))) {
            result.put(Constants.RESP_CODE, "E");
            result.put("respMsg", "请求必须参数recamt_id缺失，请检查！");
            return result;
        }

        //根据recamt_id查询ld037数据生成hls_ebank_ccb_transaction批次记录
        Long recamtId = params.getLong("recamt_id");
        Ld037 ld037 = new Ld037();
        ld037.setId(recamtId);
        ld037 = ld037Mapper.selectByPrimaryKey(ld037);
        HlsEbankCcbTransaction hlsEbankCcbTransaction = createTransaction(ld037);

//        Map<String, String> requestPublicParams = JSONObject.parseObject(params.toJSONString(), Map.class);
        // 组装requestBody并加签
        String signResult = getQrRequestParams(hlsEbankCcbTransaction);
        if (Objects.isNull(signResult)) {
            result.put(Constants.RESP_CODE, "E");
            result.put("respMsg", "收款码申请加签发生异常，请检查！");
            return result;
        }
        try {
            ObjectMapper mapper = new ObjectMapper();
            Map<String, String> signResultMap = mapper.readValue(signResult, Map.class);
            long currentTimeMills = System.currentTimeMillis() / 1000;

            // 组apiSign加密Map
            Map<String, String> apiSign = new TreeMap<>();
            apiSign.put("appid", appId);
            apiSign.put("secret", appSecret);
            apiSign.put("sign", signResultMap.get("sign"));
            apiSign.put("timestamp", "" + currentTimeMills);

            // MD5加密
            String MD5Content = SignatureUtil.getSignContent(apiSign);
            String apiSignString = MD5Utils.getMD5Content(MD5Content).toLowerCase();

            // 组request头部Map
            Map<String, String> apiHeader = new HashMap<>();
            apiHeader.put("appid", appId);
            apiHeader.put("timestamp", "" + currentTimeMills);
            apiHeader.put("apisign", apiSignString);

            // 发送HTTP post请求
            Map<String, String> response = Utils.postForEntity(url, signResult, apiHeader);
/*
            版本号	    version	String(5)	    是	固定为0.0.1
            编码方式	    encoding	String(20)	是	固定为UTF-8
            签名	sign	String(1024)	        是
            签名方法    	signMethod	String(2)	是	02：SM2
            返回码	    returnCode	String(7)	是	SUCCESS/FAIL,此字段是通信标识，非交易标识，交易是否成功需要查看respCode来判断。SUCCESS表示商户上送的报文符合规范，FAIL表示报文内的字段不符合规范，包括长度超长、非法字符、签名错误等
                    以下字段仅在returnCode为SUCCESS时返回
            商户号	    merId	String(32)	    是	请求报文中的商户号
            商户订单号   	orderId	String(32)	    是	请求报文中的商户订单号
            响应码 	    respCode	String(10)	是	业务错误码，成功为SUCCESS，失败为FAIL
                    以下字段仅在returnCode和respCode为SUCCESS时返回
            二维码	    qrCode	String(300)	    是
            平台订单号	cmbOrderId	String(32)	是	招行生成的订单号
            订单发送时间	txnTime	String(14)	    是	订单生成时间，格式为yyyyMMddHHmmss
                    以下字段仅在returnCode或respCode为FAIL时返回
            错误码	    errCode	String(32)	    是	请求处理失败的错误码信息
            应答信息	    respMsg	String(256)	    是	请求处理失败的详细描述信息
*/

            logger.info("收款码申请返回结果：" + mapper.writeValueAsString(response));

            // 返回结果验签,验证数据的正确性（安全性）
            Boolean checkResult1 = checkSign(mapper.writeValueAsString(response));
            if (!checkResult1) {
                //数据验签失败的处理，重发或者直接告诉前端发送失败
                result.put("respCode", response.get("errCode"));
            }
            //获取响应结果内容
            String success = response.get("returnCode");
            System.out.println("返回结果：" + success);
            result.put("qrcode", response.get("qrcode"));
            result.put("respMsg", response.get("respMsg"));
            //业务数据的封装。。。
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            result.put("respCode", "服务器问题");
            return result;
        }
    }

    //生成招行扫码交易业务批次数据
    private HlsEbankCcbTransaction createTransaction(Ld037 ld037) {
        HlsEbankCcbTransaction ccbTransaction = new HlsEbankCcbTransaction();
        ccbTransaction.setMerId("3089991701201D0");
        ccbTransaction.setVersion("0.0.1");
        ccbTransaction.setUserId("N003574551");
        ccbTransaction.setStatus("NEW");
        ccbTransaction.setBankType("CMB");
        ccbTransaction.setCount(1L);
        ccbTransaction.setAmount(ld037.getAmount());
        ccbTransaction.setNewDate(new Date());
        ccbTransaction.setCreatedBy(1L);
        ccbTransaction.setCreationDate(new Date());

        hlsEbankCcbTransactionMapper.insert(ccbTransaction);

        //生成单据编号 日期+id
        SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
        String dateString = formatter.format(new Date());
        ccbTransaction.setTransactionNumber(dateString + String.format("%08d", ccbTransaction.getTransactionId()));
        hlsEbankCcbTransactionMapper.updateByPrimaryKeySelective(ccbTransaction);

        return ccbTransaction;
    }

    private String getQrRequestParams(HlsEbankCcbTransaction hlsEbankCcbTransaction) {
        Map<String, String> requestPublicParams = new TreeMap<>();
        String requestStr = null;
        try {
            //公共请求参数
            requestPublicParams.put("version", version);    //版本号，固定为0.0.1(必传字段)
            requestPublicParams.put("encoding", encoding);   //编码方式，固定为UTF-8(必传)
            requestPublicParams.put("signMethod", signMethod);    //签名方法，固定为02，表示签名方式为国密(必传)

            //业务要素
            Map<String, String> requestTransactionParams = new HashMap<>();
            requestTransactionParams.put("merId", hlsEbankCcbTransaction.getMerId());   //商户号(必传)
            requestTransactionParams.put("orderId", hlsEbankCcbTransaction.getTransactionNumber()); //商户订单号(必传)
            requestTransactionParams.put("userId", hlsEbankCcbTransaction.getUserId());   //收银员
            //            requestTransactionParams.put("termId", "00774411");  //终端号
            requestTransactionParams.put("payValidTime", "900"); //支付有效时间,默认15分钟
            requestTransactionParams.put("notifyUrl", "");  //交易通知地址(必传)

            BigDecimal a1 = new BigDecimal(Double.toString(hlsEbankCcbTransaction.getAmount()));
            BigDecimal b1 = new BigDecimal(Double.toString(100));
            requestTransactionParams.put("txnAmt", a1.multiply(b1).toString());  //交易金额,单位为分(必传)
            requestTransactionParams.put("currencyCode", currencyCode);    //交易币种，默认156，目前只支持人民币（156）
            requestTransactionParams.put("body", "聚合支付测试");   //商户号(必传)

            //商户保留域
            JSONObject mchReserved = new JSONObject();
            mchReserved.put("transactionId", hlsEbankCcbTransaction.getTransactionId());
            requestTransactionParams.put("tradeScene", "OFFLINE");   //交易场景

            ObjectMapper mapper = new ObjectMapper();
            requestPublicParams.put("biz_content", mapper.writeValueAsString(requestTransactionParams));

            logger.info("收款码申请加签前的报文内容:" + mapper.writeValueAsString(requestPublicParams));

            //对待加签内容进行排序拼接
            String signContent = SignatureUtil.getSignContent(requestPublicParams);
            //加签
            requestPublicParams.put("sign", SM2Util.sm2Sign(signContent, privateKey));

            requestStr = mapper.writeValueAsString(requestPublicParams);
            logger.info("收款码申请加签后的报文内容：" + requestStr);
            return requestStr;

        } catch (Exception e) {
            logger.info("收款码申请加签发生异常：" + e.getMessage());
            e.printStackTrace();
            return requestStr;
        }
    }

    private Boolean checkSign(String string) {
        System.out.println("要验签的报文内容：" + string);
        try {
            //验签
            ObjectMapper objectMapper = new ObjectMapper();
            Map<String, String> responseBodyMap = objectMapper.readValue(string, Map.class);
            String sign = responseBodyMap.remove("sign");
            String contentStr = SignatureUtil.getSignContent(responseBodyMap);
            boolean result = SM2Util.sm2Check(contentStr, sign, publicKey);

            if (result) {
                System.out.println("报文验签成功!");
            } else {
                System.out.println("报文验签失败!");
            }
            return result;
        } catch (Exception e) {
            System.out.println("验签发生异常！");
            e.printStackTrace();
            return false;
        }
    }

    private Properties loadProp(String propPath) {
        Properties props = new Properties();
        FileInputStream is = null;

        try {
            is = new FileInputStream(propPath);
            props.load(is);
            return props;
        } catch (Exception var13) {
            LogUtil.writeErrorLog("加载配置文件失败", var13);
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException var12) {
                    ;
                }
            }

        }
        return null;
    }

}
